package org.jcg.zheng;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * the code which manipulates the shared resource is surrounded by lock() and
 * unlock() methods the current thread obtains a local and blocks all other
 * threads which are trying to lock the shared resource
 * 
 */
public class LockedService {
	private final Logger logger = LoggerFactory.getLogger(this.getClass());

	private final Lock queueLock;

	private SumTask task;

	public LockedService(Lock lock, SumTask task) {
		super();

		this.queueLock = lock;
		this.task = task;
	}

	public void foo() {
		boolean isDone = false;

		while (!isDone) {
			boolean isfree;
			try {
				isfree = queueLock.tryLock(3, TimeUnit.MINUTES);
				if (isfree) {
					try {

						// lock starts
						queueLock.lock();
						task.sum();
					} finally {
						// unlock
						queueLock.unlock();
					}
					isDone = true;
					logger.info("LockedService:foo completed.");
				} else {
					logger.info("LockedService:foo is locked by other threads. wait for 1 second");
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			} finally {
				queueLock.unlock();
			}

		}

	}

	public SumTask getTask() {
		return task;
	}

	public void setTask(SumTask task) {
		this.task = task;
	}
}
